package nl.kallestruik.darena.arenas

import nl.kallestruik.darena.exceptions.ArenaStartAbortedException
import nl.kallestruik.darena.types.arena.*
import nl.kallestruik.darena.types.border.HorizontalBorder
import nl.kallestruik.darena.types.countdown.Countdown
import org.bukkit.entity.Player
import org.bukkit.scheduler.BukkitTask
import java.util.*

data class ArenaSession(
    val originalParticipants: MutableList<Player> = mutableListOf(),
    val participants: MutableList<Player> = mutableListOf(),
    val spectators: MutableList<Player> = mutableListOf(),
    val allPlayers: MutableList<Player> = mutableListOf(),
    val completedObjective: MutableList<UUID> =  mutableListOf(),
    val deaths: MutableList<UUID> = mutableListOf(),
    val spawnRules: MutableMap<String, ProcessedArenaSpawnRule> = mutableMapOf(),
    val spawnPools: MutableMap<String, ProcessedArenaSpawnPool> = mutableMapOf(),
    val checkpoints: MutableList<ProcessedArenaCheckpoint> = mutableListOf(),
    val lastCheckpoint: MutableMap<UUID, ProcessedArenaCheckpoint> = mutableMapOf(),
    val reachedCheckpoints: MutableMap<UUID, MutableSet<String>> = mutableMapOf(),
    val lastDamage: MutableMap<UUID, UUID> = mutableMapOf(),
    val borderTop: HorizontalBorder = HorizontalBorder(),
    val borderBottom: HorizontalBorder = HorizontalBorder(),
    val finishedSectionsByPlayer: MutableMap<UUID, Int> = mutableMapOf(),
    val finishedPerSection: MutableMap<Int, Int> = mutableMapOf(),
    var startTime: Long = 0,
    var playersFinished: Int = 0,
    var timeLeft: Int = -1,
    var endTimeTask: BukkitTask? = null,
    var timerTask: BukkitTask? = null,
    var borderTask: BukkitTask? = null,
    var isInProgress: Boolean = false,
    var currentCountdown: Countdown? = null,
) {

    fun processSpawnRules(spawns: Map<String, ArenaSpawn>, rules: Map<String, ArenaSpawnRule>, loadouts: Map<String, ArenaLoadout>) {
        for (rule in rules) {
            val list = mutableListOf<ArenaSpawn>()
            for (spawn in rule.value.spawns) {
                if (spawn in spawns)
                    list.add(spawns[spawn]!!)
                else
                    throw ArenaStartAbortedException("Spawn $spawn is used in spawn rule ${rule.key} but is not defined. Please resolve this issue before trying to start this arena.")
            }

            list.shuffle()

            if (!loadouts.containsKey(rule.value.loadout) && rule.value.loadout != "none")
                throw ArenaStartAbortedException("Loadout ${rule.value.loadout} is used in spawn rule ${rule.key} but is not defined. Please resolve this issue before trying to start this arena.")

            spawnRules[rule.key] = ProcessedArenaSpawnRule(rule.value, list, loadouts[rule.value.loadout])
        }
    }

    fun processSpawnPools(pools: Map<String, ArenaSpawnPool>) {
        for (pool in pools) {
            val list = mutableListOf<ProcessedArenaSpawnRule>()
            for (rule in pool.value.rules) {
                if (rule in spawnRules)
                    list.add(spawnRules[rule]!!)
                else
                    throw ArenaStartAbortedException("Spawn rule $rule is used in spawn pool ${pool.key} but is not defined. Please resolve this issue before trying to start this arena.")
            }

            list.shuffle()

            spawnPools[pool.key] = ProcessedArenaSpawnPool(pool.value.label, pool.value.times, list)
        }
    }

    fun processCheckpoints(checkpoints: List<ArenaCheckpoint>) {
        for (checkpoint in checkpoints) {
            val spawnRule = spawnRules[checkpoint.spawnRule]
            val spawnPool = spawnPools[checkpoint.spawnPool]

            this.checkpoints.add(ProcessedArenaCheckpoint(checkpoint, spawnRule, spawnPool))
        }
    }
}
