package nl.kallestruik.darena.listeners

import net.kyori.adventure.text.Component
import nl.kallestruik.darena.managers.EditManager
import nl.kallestruik.darena.types.EditSetting
import nl.kallestruik.darena.types.arena.ArenaSpawn
import nl.kallestruik.dlib.gui.chestGUI
import nl.kallestruik.dlib.gui.textInputDialog
import org.bukkit.Material
import org.bukkit.entity.ArmorStand
import org.bukkit.entity.Player
import org.bukkit.event.EventHandler
import org.bukkit.event.Listener
import org.bukkit.event.entity.EntityDamageByEntityEvent
import org.bukkit.event.entity.EntityPlaceEvent
import org.bukkit.event.player.PlayerInteractAtEntityEvent
import org.bukkit.plugin.Plugin

class EditListener(
    val editManager: EditManager,
    val plugin: Plugin
): Listener {

    @EventHandler
    fun onEntityPlace(event: EntityPlaceEvent) {
        if (!editManager.shouldProcessEvent(event.player, EditSetting.SPAWN_PLACING))
            return

        val entity = event.entity

        if (entity !is ArmorStand)
            return

        val loc = entity.location

        val spawn = ArenaSpawn(loc.x, loc.y, loc.z, loc.yaw, loc.pitch)

        textInputDialog(event.player!!, plugin) {
            prompt = Component.text("Spawn name?")

            item {
                material = Material.PAPER
                amount = 1
            }

            onFinish { name ->
                if (editManager.addSpawn(name, spawn)) {
                    event.player?.sendMessage("Added spawn $name")
                } else {
                    event.player?.sendMessage("Adding spawn failed. Is the name already in use?")
                }
                event.entity.remove()
            }

            onCancel {
                event.player?.sendMessage("Canceled adding spawn")
                event.entity.remove()
            }
        }
    }

    @EventHandler
    fun onEntityDamage(event: EntityDamageByEntityEvent) {
        if (event.damager !is Player)
            return

        val player = event.damager as Player

        if (!editManager.shouldProcessEvent(player, EditSetting.ALWAYS)) {
            return
        }

        if (event.entity is ArmorStand) {
            entityDamageArmorStand(player, event)
        }
    }

    private fun entityDamageArmorStand(player: Player, event: EntityDamageByEntityEvent) {
        if (!event.entity.hasMetadata("ArenaSpawn"))
            return

        // This cast should be safe since we should be the only ones editing this.
        val spawnName = event.entity.getMetadata("ArenaSpawn")[0].value() as String

        event.isCancelled = true

        editManager.removeSpawnGUI(player, spawnName)
    }

    @EventHandler
    fun onPlayerInteractAtEntity(event: PlayerInteractAtEntityEvent) {
        if (!editManager.shouldProcessEvent(event.player, EditSetting.ALWAYS)) {
            return
        }

        if (event.rightClicked is ArmorStand) {
            playerInteractArmorstand(event.player, event)
        }
    }

    private fun playerInteractArmorstand(player: Player, event: PlayerInteractAtEntityEvent) {
        if (!event.rightClicked.hasMetadata("ArenaSpawn"))
            return

        // This cast should be safe since we should be the only ones editing this.
        val spawnName = event.rightClicked.getMetadata("ArenaSpawn")[0].value() as String

        val spawn = editManager.getSpawn(spawnName)

        event.isCancelled = true

        chestGUI(player, plugin) {
            title = Component.text("Options for: $spawnName")
            rows = 3

            //TODO: Any more actions to add to a spawn?

            //   0 1 2 3 4 5 6 7 8
            // 0 b b b b I b b b b
            // 1 b x R x x x D x b
            // 2 b x x x x x x x b
            // 3 b b b b b b b b b

            border {
                material = Material.LIGHT_BLUE_STAINED_GLASS_PANE
            }

            entry {
                x = 4
                y = 0

                name = Component.text("Information")
                material = Material.PAPER

                lore(Component.empty())
                lore(Component.text("x: ${spawn.x}"))
                lore(Component.text("y: ${spawn.y}"))
                lore(Component.text("z: ${spawn.z}"))
                lore(Component.text("yaw: ${spawn.yaw}"))
                lore(Component.text("pitch: ${spawn.pitch}"))
            }

            entry {
                x = 2
                y = 1

                name = Component.text("Add to rule")
                material = Material.LIME_DYE

                onLeftClick {
                    editManager.addSpawnToRuleGUI(player, spawnName)
                }
            }

            entry {
                x = 6
                y = 1

                name = Component.text("Delete")
                material = Material.BARRIER

                onLeftClick {
                    editManager.removeSpawnGUI(player, spawnName)
                }
            }
        }
    }
}