package nl.kallestruik.darena.types.arena

import kotlinx.serialization.KSerializer
import kotlinx.serialization.Serializable
import kotlinx.serialization.descriptors.PrimitiveKind
import kotlinx.serialization.descriptors.PrimitiveSerialDescriptor
import kotlinx.serialization.descriptors.SerialDescriptor
import kotlinx.serialization.encoding.Decoder
import kotlinx.serialization.encoding.Encoder
import nl.kallestruik.darena.util.Logger
import nl.kallestruik.dlib.config.annotations.DefaultString

@Serializable
data class ArenaAllowRule(
    @Serializable(RegexSerializer::class)
    var rule: Regex,
    @DefaultString("BLOCK")
    var type: Type,
) {
    fun match(toMatch: String): AllowRuleResult {
        Logger.trace(ArenaAllowRule::class, "match(toMatch: $toMatch)")
        if (rule.matches(toMatch)) {
            Logger.debug(ArenaAllowRule::class, "Rule matches")
            return when (type) {
                Type.ALLOW -> AllowRuleResult.ALLOWED
                Type.BLOCK -> AllowRuleResult.BLOCKED
            }
        }

        return AllowRuleResult.NO_MATCH
    }

    object RegexSerializer: KSerializer<Regex> {
        override val descriptor: SerialDescriptor = PrimitiveSerialDescriptor("Regex", PrimitiveKind.STRING)

        override fun serialize(encoder: Encoder, value: Regex) {
            encoder.encodeString(value.pattern)
        }

        override fun deserialize(decoder: Decoder): Regex {
           return Regex(decoder.decodeString())
        }
    }

    enum class Type {
        ALLOW,
        BLOCK;
    }
}

enum class AllowRuleResult {
    NO_MATCH,
    ALLOWED,
    BLOCKED,
}
