package nl.kallestruik.darena.types.arena

import kotlinx.serialization.Serializable
import nl.kallestruik.darena.util.Logger
import nl.kallestruik.dlib.config.annotations.DefaultInt
import nl.kallestruik.dlib.config.annotations.Description
import nl.kallestruik.dlib.config.annotations.MinInt
import org.bukkit.NamespacedKey
import org.bukkit.enchantments.Enchantment
import org.bukkit.inventory.ItemStack

@Serializable
data class ArenaEnchantment(
    @Description(["The name of the enchantment.",
        "", "Valid values: The id of the enchantment as shown in the /enchant command without the minecraft:",
        "For example: sharpness, knockback, power"])
    val name: String,
    @Description(["The level of the enchantment. Allows over enchanting."])
    @MinInt(1)
    @DefaultInt(1)
    val level: Int,
) {
    fun addToItem(stack: ItemStack) {
        Logger.trace(ArenaEnchantment::class, "addToItem(stack: $stack)")
        stack.addUnsafeEnchantment(Enchantment.getByKey(NamespacedKey.minecraft(name.lowercase()))!!, level)

    }
}
