package nl.kallestruik.darena.types.border

import kotlinx.serialization.KSerializer
import kotlinx.serialization.Serializable
import kotlinx.serialization.SerializationException
import kotlinx.serialization.Transient
import kotlinx.serialization.descriptors.PrimitiveKind
import kotlinx.serialization.descriptors.PrimitiveSerialDescriptor
import kotlinx.serialization.descriptors.SerialDescriptor
import kotlinx.serialization.encoding.Decoder
import kotlinx.serialization.encoding.Encoder
import org.bukkit.plugin.java.JavaPlugin

@Serializable(AbstractBorderNode.BorderScriptSerializer::class)
abstract class AbstractBorderNode {
    @Transient
    var next: AbstractBorderNode? = null

    abstract fun execute(border: AbstractBorder, plugin: JavaPlugin)

    abstract fun toCommand(): String

    protected fun runNext(border: AbstractBorder, plugin: JavaPlugin) {
        next?.execute(border, plugin)
    }

    open fun cancel() {
        next?.cancel()
    }

    object BorderScriptSerializer: KSerializer<AbstractBorderNode> {
        override val descriptor: SerialDescriptor = PrimitiveSerialDescriptor("BorderScriptNode", PrimitiveKind.STRING)

        override fun serialize(encoder: Encoder, value: AbstractBorderNode) {
            encoder.encodeString(value.toCommand())
        }

        override fun deserialize(decoder: Decoder): AbstractBorderNode {
            return createNode(decoder.decodeString())
        }

        private fun createNode(command: String): AbstractBorderNode {
            val parts = command.split(" ", limit = 2)
            return when (parts[0].uppercase()) {
                "CENTER" -> BorderNodeCenter.fromParameters(parts[1])
                "SET" -> BorderNodeSet.fromParameters(parts[1])
                "TRANSITION" -> BorderNodeTransition.fromParameters(parts[1])
                "WAIT" -> BorderNodeWait.fromParameters(parts[1])
                else -> throw SerializationException("${parts[0]} is not a valid border script keyword.")
            }
        }
    }
}