package nl.kallestruik.darena.util

import org.bukkit.Color
import org.bukkit.Particle
import org.bukkit.World

object RenderUtil {
    private const val AABB_PARTICLES_PER_BLOCK = 2
    private const val PLANE_PARTICLES_PER_BLOCK = 1

    fun drawXZPlane(world: World,
                  lowerX: Int, lowerZ: Int,
                  upperX: Int, upperZ: Int,
                  y: Int, color: Color) {

        for (x in upperX downTo lowerX) {
            for (z in upperZ downTo lowerZ) {
                world.spawnParticle(
                    Particle.REDSTONE,
                    x + 0.5,
                    y + 0.5,
                    z + 0.5,
                    PLANE_PARTICLES_PER_BLOCK,
                    0.5,
                    0.0,
                    0.5,
                    Particle.DustOptions(color, 1.0F)
                )
            }
        }
    }

    fun drawAABB(world: World,
                 lowerX: Int, lowerY: Int, lowerZ: Int,
                 upperX: Int, upperY: Int, upperZ: Int,
                 color: Color) {
        //    h--g
        //   /| /|
        //  e--f |    y   z
        //  | d|-c    |  /
        //  |/ |/     | /
        //  a--b      0 ---> x
        //
        // a = lower
        // g = upper


        // AB
        for (x in upperX downTo lowerX) {
            world.spawnParticle(
                Particle.REDSTONE,
                x + 0.5,
                lowerY.toDouble(),
                lowerZ.toDouble(),
                AABB_PARTICLES_PER_BLOCK,
                0.5,
                0.0,
                0.0,
                Particle.DustOptions(color, 1.0F)
            )
        }

        // AD
        for (z in upperZ downTo lowerZ) {
            world.spawnParticle(
                Particle.REDSTONE,
                lowerX.toDouble(),
                lowerY.toDouble(),
                z + 0.5,
                AABB_PARTICLES_PER_BLOCK,
                0.0,
                0.0,
                0.5,
                Particle.DustOptions(color, 1.0F)
            )
        }

        // AE
        for (y in upperY downTo lowerY) {
            world.spawnParticle(
                Particle.REDSTONE,
                lowerX.toDouble(),
                y + 0.5,
                lowerZ.toDouble(),
                AABB_PARTICLES_PER_BLOCK,
                0.0,
                0.5,
                0.0,
                Particle.DustOptions(color, 1.0F)
            )
        }

        // GH
        for (x in upperX downTo lowerX) {
            world.spawnParticle(
                Particle.REDSTONE,
                x + 0.5,
                upperY + 1.0,
                upperZ + 1.0,
                AABB_PARTICLES_PER_BLOCK,
                0.5,
                0.0,
                0.0,
                Particle.DustOptions(color, 1.0F)
            )
        }

        // FG
        for (z in upperZ downTo lowerZ) {
            world.spawnParticle(
                Particle.REDSTONE,
                upperX + 1.0,
                upperY + 1.0,
                z + 0.5,
                AABB_PARTICLES_PER_BLOCK,
                0.0,
                0.0,
                0.5,
                Particle.DustOptions(color, 1.0F)
            )
        }

        // CG
        for (y in upperY downTo lowerY) {
            world.spawnParticle(
                Particle.REDSTONE,
                upperX + 1.0,
                y + 0.5,
                upperZ + 1.0,
                AABB_PARTICLES_PER_BLOCK,
                0.0,
                0.5,
                0.0,
                Particle.DustOptions(color, 1.0F)
            )
        }

        // BF
        for (y in upperY downTo lowerY) {
            world.spawnParticle(
                Particle.REDSTONE,
                upperX + 1.0,
                y + 0.5,
                lowerZ.toDouble(),
                AABB_PARTICLES_PER_BLOCK,
                0.0,
                0.5,
                0.0,
                Particle.DustOptions(color, 1.0F)
            )
        }

        // DH
        for (y in upperY downTo lowerY) {
            world.spawnParticle(
                Particle.REDSTONE,
                lowerX.toDouble(),
                y + 0.5,
                upperZ + 1.0,
                AABB_PARTICLES_PER_BLOCK,
                0.0,
                0.5,
                0.0,
                Particle.DustOptions(color, 1.0F)
            )
        }

        // AF
        for (x in upperX downTo lowerX) {
            world.spawnParticle(
                Particle.REDSTONE,
                x + 0.5,
                upperY + 1.0,
                lowerZ.toDouble(),
                AABB_PARTICLES_PER_BLOCK,
                0.5,
                0.0,
                0.0,
                Particle.DustOptions(color, 1.0F)
            )
        }

        // AH
        for (z in upperZ downTo lowerZ) {
            world.spawnParticle(
                Particle.REDSTONE,
                lowerX.toDouble(),
                upperY + 1.0,
                z + 0.5,
                AABB_PARTICLES_PER_BLOCK,
                0.0,
                0.0,
                0.5,
                Particle.DustOptions(color, 1.0F)
            )
        }

        // BC
        for (z in upperZ downTo lowerZ) {
            world.spawnParticle(
                Particle.REDSTONE,
                upperX + 1.0,
                lowerY.toDouble(),
                z + 0.5,
                AABB_PARTICLES_PER_BLOCK,
                0.0,
                0.0,
                0.5,
                Particle.DustOptions(color, 1.0F)
            )
        }

        // CD
        for (x in upperX downTo lowerX) {
            world.spawnParticle(
                Particle.REDSTONE,
                x + 0.5,
                lowerY.toDouble(),
                upperZ + 1.0,
                AABB_PARTICLES_PER_BLOCK,
                0.5,
                0.0,
                0.0,
                Particle.DustOptions(color, 1.0F)
            )
        }
    }
}